@@ -8,7 +8,7 @@ from functools import wraps
 
 
 __all__ = ['Dispatcher', 'command', 'dispatch']
-__version__ = '3.0'
+__version__ = '3.1'
 __author__ = 'Alexander Solovyov'
 __email__ = 'alexander@solovyov.net'
 
@@ -62,7 +62,7 @@ class Dispatcher(object):
         return table
 
     def command(self, options=None, usage=None, name=None, shortlist=False,
-                hide=False):
+                hide=False, aliases=()):
         '''Decorator to mark function to be used as command for CLI.
 
         Usage::
@@ -95,6 +95,7 @@ class Dispatcher(object):
            only with multiple subcommands
          - ``hide``: if command should be hidden from help listing. Used only
            with multiple subcommands, overrides ``shortlist``
+         - ``aliases``: list of aliases for command
 
         If defined, options should be a list of 4-tuples in format::
 
@@ -123,10 +124,14 @@ class Dispatcher(object):
             else:
                 usage_ = usage
             prefix = hide and '~' or (shortlist and '^' or '')
-            self._cmdtable[prefix + name_] = (func, options_, usage_)
+            cmdname = prefix + name_
+            if aliases:
+                cmdname = cmdname + '|' + '|'.join(aliases)
+            self._cmdtable[cmdname] = (func, options_, usage_)
 
             def help_func(name=None):
-                return help_cmd(func, replace_name(usage_, sysname()), options_)
+                return help_cmd(func, replace_name(usage_, sysname()), options_,
+                                aliases)
 
             def command(argv=None):
                 for o in self.globaloptions:
@@ -173,7 +178,7 @@ class Dispatcher(object):
 
         if options.pop('help', False):
             return 'help', self.cmdtable['help'][0], [cmd], {}
-        elif not cmd:
+        if not cmd:
             return 'help', self.cmdtable['help'][0], ['shortlist'], {}
 
         return cmd, func, args, options
@@ -239,7 +244,7 @@ dispatch.__doc__ = Dispatcher.dispatch.__doc__
 def help_(cmdtable, globalopts):
     '''Help generator for a command table
     '''
-    def help_inner(name=None):
+    def help_inner(name=None, **opts):
         '''Show help for a given help topic or a help overview
 
         With no arguments, print a list of commands with short help messages.
@@ -279,10 +284,11 @@ def help_(cmdtable, globalopts):
         aliases, (cmd, options, usage) = findcmd(name, cmdtable)
         return help_cmd(cmd,
                         replace_name(usage, sysname() + ' ' + aliases[0]),
-                        options + globalopts)
+                        options + globalopts,
+                        aliases[1:])
     return help_inner
 
-def help_cmd(func, usage, options):
+def help_cmd(func, usage, options, aliases):
     '''show help for given command
 
     - ``func``: function to generate help for (``func.__doc__`` is taken)
@@ -302,7 +308,7 @@ def help_cmd(func, usage, options):
     ...          'daemonize process'),
     ...         ('', 'pid-file', '',
     ...          'name of file to write process ID to')]
-    >>> help_cmd(test, 'test [-l HOST] [NAME]', opts)
+    >>> help_cmd(test, 'test [-l HOST] [NAME]', opts, ())
     test [-l HOST] [NAME]
     <BLANKLINE>
     that's a test command
@@ -317,6 +323,8 @@ def help_cmd(func, usage, options):
         --pid-file   name of file to write process ID to
     '''
     write(usage + '\n')
+    if aliases:
+        write('\naliases: ' + ', '.join(aliases) + '\n')
     doc = pretty_doc_string(func)
     write('\n' + doc.strip() + '\n\n')
     if options:
@@ -417,8 +425,9 @@ def parse(args, options):
             try:
                 k, v = val.split('=')
             except ValueError:
-                raise ParseError(name, "wrong definition: '%s' "
-                                 "(should be in format KEY=VALUE)" % val)
+                raise getopt.GetoptError(
+                    "wrong definition: '%s' (should be in format KEY=VALUE)"
+                    % val)
             state[name][k] = v
         elif t in (types.NoneType, types.BooleanType):
             state[name] = not defmap[name]
@@ -517,12 +526,11 @@ def guess_options(func):
     '''
     args, _, _, defaults = inspect.getargspec(func)
     for name, option in zip(args[-len(defaults):], defaults):
-        try:
-            sname, default, hlp = option[:3]
-            completer = option[3] if len(option) > 3 else None
-            yield (sname, name_from_python(name), default, hlp, completer)
-        except TypeError:
-            pass
+        if not isinstance(option, tuple):
+            continue
+        sname, default, hlp = option[:3]
+        completer = option[3] if len(option) > 3 else None
+        yield (sname, name_from_python(name), default, hlp, completer)
 
 def guess_usage(func, options):
     '''Get usage definition for a function
@@ -531,13 +539,13 @@ def guess_usage(func, options):
     if options:
         usage.append('[OPTIONS]')
     arginfo = inspect.getargspec(func)
+    optnames = [x[1] for x in options]
+    nonoptional = len(arginfo.args) - len(arginfo.defaults or ())
 
-    argnum = len(arginfo.args or ()) - len(options)
-    nonoptional = len(arginfo.args or ()) - len(arginfo.defaults or ())
-
-    for i in xrange(argnum):
-        usage.append((i > nonoptional - 1 and '[%s]' or '%s') %
-                     arginfo.args[i].upper())
+    for i, arg in enumerate(arginfo.args):
+        if arg not in optnames:
+            usage.append((i > nonoptional - 1 and '[%s]' or '%s')
+                         % arg.upper())
 
     if arginfo.varargs:
         usage.append('[%s ...]' % arginfo.varargs.upper())
